import json
from asyncio.subprocess import create_subprocess_exec
from os import path
from subprocess import PIPE, run as run_subprocess
from tempfile import TemporaryDirectory
from typing import Any

# executes a function from the specified source file
r_source = """
args = commandArgs(trailingOnly = TRUE)
source_file <- args[1]
function_name <- args[2]
function_arguments_file <- args[3]
output_file <- args[4]

source(source_file)
function_arguments <- list()
if (file.exists(function_arguments_file))
    function_arguments <- as.list(jsonlite::read_json(function_arguments_file))
output <- do.call(function_name, function_arguments)
jsonlite::write_json(output, output_file, auto_unbox=TRUE, force = TRUE, na = "null")
"""


def run(
    source_file: str,
    function_name: str,
    arguments: Any = None,
    process_arguments: dict = {},
) -> Any:
    """
    Calls an R function from a source file and returns its output. If the
    function does not return a serializable value, stdout is returned.

    Keyword arguments:
    - source_file -- Path to the R source file
    - function_name -- Name of the R function to execute
    - arguments -- Arguments to apply to the R function - use a list for positional
      arguments or a dict for named arguments.
    - process_arguments -- Additional arguments passed to the R subprocess

    Returns:
        The R function's output
    """

    with TemporaryDirectory() as temp_dir:
        r_file = path.join(temp_dir, "source.R")
        input_file = path.join(temp_dir, "input.json")
        output_file = path.join(temp_dir, "output.json")

        with open(r_file, "w") as fp:
            fp.write(r_source)

        if arguments is not None:
            with open(input_file, "w") as fp:
                json.dump(arguments, fp)

        subprocess_arguments = {
            "check": True,
            "stdout": PIPE,
            "stderr": PIPE,
            **process_arguments,
        }
        stdout = run_subprocess(
            [
                "Rscript",
                r_file,
                source_file,
                function_name,
                input_file,
                output_file,
            ],
            **subprocess_arguments
        )

        if path.exists(output_file):
            with open(output_file) as fp:
                return json.load(fp)
        else:
            return stdout


async def run_async(
    source_file: str,
    function_name: str,
    arguments: Any = None,
    process_arguments: dict = {},
) -> Any:
    """
    Calls an R function from a source file asynchronously and returns its output.
    If the function does not return a serializable value, stdout is returned.

    Keyword arguments:
    - source_file -- Path to the R source file
    - function_name -- Name of the R function to execute
    - arguments -- Arguments to apply to the R function - use a list for positional
      arguments or a dict for named arguments.
    - process_arguments -- Additional arguments passed to the R subprocess

    Returns:
        The R function's output
    """

    with TemporaryDirectory() as temp_dir:
        r_file = path.join(temp_dir, "source.R")
        input_file = path.join(temp_dir, "input.json")
        output_file = path.join(temp_dir, "output.json")

        with open(r_file, "w") as fp:
            fp.write(r_source)

        if arguments is not None:
            with open(input_file, "w") as fp:
                json.dump(arguments, fp)

        subprocess_arguments = {"stdout": PIPE, "stderr": PIPE, **process_arguments}
        proc = await create_subprocess_exec(
            "Rscript",
            r_file,
            source_file,
            function_name,
            input_file,
            output_file,
            **subprocess_arguments
        )
        returncode = await proc.wait()
        stdout, stderr = await proc.communicate()

        if returncode != 0:
            raise Exception(
                {
                    "returncode": returncode,
                    "output": stdout,
                    "stdout": stdout,
                    "stderr": stderr,
                }
            )

        if path.exists(output_file):
            with open(output_file) as fp:
                return json.load(fp)
        else:
            return stdout


def create(source_file: str, function_name: str):
    """
    Creates a python function bound to an R function from a source file. The function
    may be called with either positional or named arguments, but not both.

    Keyword arguments:
    - source_file -- Path to the R source file
    - function_name -- Name of the R function to execute

    Returns:
        A bound function
    """

    return lambda *args, **kwargs: run(
        source_file, function_name, args if args else kwargs
    )


def create_async(source_file: str, function_name: str):
    """
    Creates an async python function bound to an R function from a source file. The function
    may be called with either positional or named arguments, but not both.

    Keyword arguments:
    - source_file -- Path to the R source file
    - function_name -- Name of the R function to execute

    Returns:
        A bound function
    """

    return lambda *args, **kwargs: run_async(
        source_file, function_name, args if args else kwargs
    )



import unittest
from os import path
from subprocess import CalledProcessError
from r_functions import create, run


class TestR(unittest.TestCase):
    def setUp(self):
        self.test_file = path.join(path.dirname(__file__), "test.R")

    def test_create_add(self):
        params = {"a": 2, "b": 3}
        add = create(self.test_file, "add")
        output = add(**params)
        self.assertEqual(5, output)

    def test_create_reverse(self):
        params = [["a", "b", "c"]]
        reverse = create(self.test_file, "reverse")
        output = reverse(*params)
        self.assertEqual(["c", "b", "a"], output)

    def test_create_exception(self):
        throw_exception = create(self.test_file, "throw_exception")
        self.assertRaises(CalledProcessError, throw_exception)

    def test_run_add(self):
        params = {"a": 2, "b": 3}
        output = run(self.test_file, "add", params)
        self.assertEqual(5, output)

    def test_run_reverse(self):
        params = [["a", "b", "c"]]
        output = run(self.test_file, "reverse", params)
        self.assertEqual(["c", "b", "a"], output)

    def test_run_exception(self):
        self.assertRaises(CalledProcessError, run, self.test_file, "throw_exception")


if __name__ == "__main__":
    unittest.main()



import asyncio
import sys
import unittest
from os import path
from r_functions import create_async, run_async

if sys.platform == "win32":
    asyncio.set_event_loop(asyncio.ProactorEventLoop())


class TestRAsync(unittest.TestCase):
    def setUp(self):
        self.loop = asyncio.get_event_loop()
        self.test_file = path.join(path.dirname(__file__), "test.R")

    def test_create_add(self):
        params = {"a": 2, "b": 3}
        add = create_async(self.test_file, "add")
        output = self.loop.run_until_complete(add(**params))
        self.assertEqual(5, output)

    def test_create_reverse(self):
        params = [["a", "b", "c"]]
        reverse = create_async(self.test_file, "reverse")
        output = self.loop.run_until_complete(reverse(*params))
        self.assertEqual(["c", "b", "a"], output)

    def test_create_exception(self):
        throw_exception = create_async(self.test_file, "throw_exception")
        self.assertRaises(
            Exception,
            lambda: self.loop.run_until_complete(throw_exception()),
        )

    def test_run_add(self):
        params = {"a": 2, "b": 3}
        output = self.loop.run_until_complete(run_async(self.test_file, "add", params))
        self.assertEqual(5, output)

    def test_run_reverse(self):
        params = [["a", "b", "c"]]
        output = self.loop.run_until_complete(
            run_async(self.test_file, "reverse", params)
        )
        self.assertEqual(["c", "b", "a"], output)

    def test_run_exception(self):
        self.assertRaises(
            Exception,
            lambda: self.loop.run_until_complete(
                run_async(self.test_file, "throw_exception")
            ),
        )


if __name__ == "__main__":
    unittest.main()



from setuptools import setup

with open("README.md") as fh:
    long_description = fh.read()

setup(
    name="r-functions",
    version="1.0.3",
    author="re.public",
    author_email="re.public@outlook.com",
    description="A library for running R functions from a source file",
    keywords="R, function, wrapper, library, async",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/park-brian/r-functions",
    py_modules=["r_functions"],
    test_suite="test",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.6",
    zip_safe=True,
)
